/*
  fnc_general
  
  dates: 00.10 re-written by Karl J. Ellefsen

  description: In this file are general c functions that are called by
  the "fd_bh".

*/

/*
      function file_open2

      - author: K. J. Ellefsen

      - description: opens a file

*/

#include <stdio.h>   /* fprintf, fopen */
#include <stddef.h>  /* NULL */
#include <stdlib.h>  /* exit */

FILE *file_open2 ( char *name, char *mode )

{

	FILE *ptr ;

	ptr = fopen ( name, mode ) ;

	if( NULL == ptr ) {

		fprintf ( stderr, "Error opening file %s\n", name );

		exit ( EXIT_FAILURE ) ;
	}

	return ( ptr ) ;
}

/*
      function mem_alloc2

      - author: K. J. Ellefsen

      - description: allocates memory for an array

*/


#include <stdio.h>   /* fprintf */
#include <stddef.h>  /* NULL */
#include <stdlib.h>  /* malloc, exit */

void *mem_alloc2 ( size_t nbytes, char array_name[] )

{
	char *address ;

	address = malloc ( nbytes ) ;

	if ( address == NULL ) {

		fprintf( stderr,"Error occurred while allocating memory for array %s\n", array_name);

		exit ( EXIT_FAILURE ) ;
	}

	return ( address ) ;
}


/*
  read an array of floating point numbers to a file

  fp = file pointer
  array = pointer to array of floats
  n = number of floats to be read

  The function returns the number of floats read.

*/

#include<stdio.h>  /* fread, fprintf, ferror, feof */

long int file_read_float( FILE *fp, float *array, long int n )

{
	long int i ;

	i = fread( array, sizeof( float ), n, fp ) ;

	if( i != n ) {

		fprintf( stderr, "ERROR: Function file_read_float.\n" ) ;
		fprintf( stderr, "       Number of floats read = %ld\n", i ) ;
		fprintf( stderr, "       ferror = %d\n", ferror(fp) ) ;
		fprintf( stderr, "       feof = %d\n", feof(fp) ) ;

		return(NULL) ;
	}

	return(i) ;
}


/*
  write an array of floats to a file

  fp = file pointer
  array = pointer to array of floats
  n = number of floats to be written

  The function returns the number of floats written.

*/

#include<stdio.h>           /* fwrite, fprintf, stderr */
#include<stddef.h>          /* NULL */
                            /* ferror is not ANSI C */

long int file_write_float( FILE *fp, float *array, long int n )

{
	long int i ;

	i = fwrite( array, sizeof( float ), n, fp ) ;

	if( i != n ) {

		fprintf( stderr, "ERROR: Function file_write_float.\n" ) ;
		fprintf( stderr, "       Number of floats written = %ld\n", i ) ;
		fprintf( stderr, "       ferror = %d\n", ferror(fp) ) ;

		return(NULL) ;
	}

	return(i) ;
}

/*
  write an array of long integers to a file

  fp = file pointer
  array = pointer to array of long integers
  n = number of long integers to be written

  The function returns the number of long integers written.

*/

#include<stdio.h>           /* fwrite, fprintf, stderr */
#include<stddef.h>          /* NULL */
                            /* ferror is not ANSI C */

long int file_write_long( FILE *fp, long int *array, long int n )

{
	long int i ;

	i = fwrite( array, sizeof( long int ), n, fp ) ;

	if( i != n ) {

		fprintf( stderr, "ERROR: Function file_write_long.\n" ) ;
		fprintf( stderr, "       Number of long integers written = %ld\n", i ) ;
		fprintf( stderr, "       ferror = %d\n", ferror(fp) ) ;

		return(NULL) ;
	}

	return(i) ;
}


/* 

  Determine whether a point is inside or outside a polygon. 

  algorithm 112, CAMC, position of point relative to polygon
  written by M. Shimrat
  All variable names are identical to those given in the algol source listing. 
  The vertices of the polygon are in clockwise or counterclockwise order, and
  the first vertex has been copied and put into the last location in the arrays,
  *x and *y.

  n = number of vertices in the polygon
  x0,y0 = x,y location of point to be evaluated
  *x,*y = two arrays containing the x,y locations of the vertices of the polygon.
     The arrays consist of n+1 points.

  If the point is inside the polygon, status is /= 0; otherwise, status is 0.

*/

void poly_pts( long int *n, float *x, float *y, float *x0, float *y0, long int *status )

{

	long int i, b ;
	float z1,z2;
	
	b = 1;
	for (i=0;i<(*n);i++) {
		
		z1 = ( *y0 - y[i] ) * ( x[i+1] - x[i] );
		z2 = y[i+1] - y[i];
		
		if ( ( (*y0<=y[i]) == (*y0>y[i+1]) ) ){
		 	if ( (*x0 - x[i] - z1/z2)<0.0 ) b=!b;
		}
	}
	b = !b;

	*status = b ;
}


/*
  argck

  argc is the number of commmand line arguments (including the command itself)
  argv are the command line arguments (including the command itself)
  arg is simply a string

  argck determines whether arg matches any command line arguments (argv)
  If so, the function returns the number corresponding to the argument.
  If not, the function returns 0.

  This function was given to me by Greg Duckworth.

*/

#include<string.h>

long int argck( int argc, char **argv, char *arg )

{
	long int i;

	for( i = 1 ; i < argc ; i++ )
	  if( !strcmp( argv[i], arg ) )
	    return(i) ;

	return(0) ;
}

/*
  function: round_off

  - author: K. J. Ellefsen

  - description: rounds a real number (double) to an integer (long int)

*/

#include <math.h> /* modf */

long int round_off( double x ) 

{
	long int m ;

	double n, f ;

	f = modf( x, &n ) ;

	m = (long int) n ;

	if ( 0.0 <= x ) {

		if ( f < 0.5 )

		  return ( m ) ;

		else

		  return ( m + 1 ) ;

	}
	else {

		if ( f > -0.5 )
		  
		  return ( m ) ;

		else

		  return ( m - 1 ) ;

	}

}


/*
  polygon_area

  Function that computes the area of a polygon.

  input:
  x = x coordinates of the vertices of the polygon
  y = y coordinates of the vertices of the polygon
  n = number of vertices

  x and y are vectors of size n

  The returned value of the function is the computed area.

*/

double polygon_area ( float x[], float y[], long int n )

{
	long int i ;

	double sum ;

	sum = 0. ;

	for( i = 0 ; i < n - 1 ; ++ i ) 
	  sum += ( y[i+1] + y[i] ) * ( x[i+1] - x[i] ) / 2. ;

	sum += ( y[0] + y[n-1] ) * ( x[0] - x[n-1] ) / 2. ;

	/* the sum can be negative, if the points are in counter-clockwise
	   order */

	if( sum < 0. ) sum = - sum ;

	return( sum ) ;

}

